/* rotatelightsGLUT.c
 * Exemple d'utilisation de plusieurs sources de lumires en rotation
 * autour d'un objet.
 * Touches:
 *	ESC - Quitter
 *	p - Mode pause
 *	r - Activation/Arrt de la lumire rouge
 *	g - Activation/Arrt de la lumire verte
 *	b - Activation/Arrt de la lumire bleue
 *	s - Activation/Arrt de la rotation de l'objet
 *	1 - l'objet est un cube
 *	2 - l'objet est une sphre
 *	3 - l'objet est un tore
 *
 * Auteur: VOURIOT Thierry
 * Web: http://thierry.vouriot.free.fr
 * Email: thierry.vouriot@free.fr
 * Date: 22 juillet 2002
 */

#include <GL/glut.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* Caractristiques de la fentre et de l'cran */
#define TITLE "RotateLightsGLUT (Vouriot Thierry)"	// titre du programme
#define WINDOW_WIDTH 350	// largeur de la fentre
#define WINDOW_HEIGHT 350	// largeur de la fentre
#define	WINDOW_POS_X 100	// position x de la fentre
#define WINDOW_POS_Y 100	// position y de la fentre
#define FULLSCREEN_WIDTH 640	// largeur en mode plein cran
#define FULLSCREEN_HEIGHT 480	// hauteur en mode plein cran
#define FULLSCREEN_DEPTH 16	// bit par pixels en mode plen cran

/* Dfinitions pour Windows */
#ifdef _WIN32
#include <windows.h>
#pragma comment( lib, "opengl32.lib" )			// recherche de OpenGL32.lib pendant l'dition de liens
#pragma comment( lib, "glu32.lib" )			// recherche de Glu32.lib pendant l'dition de liens
#pragma comment( lib, "glut32.lib" )			// recherche de Glut32.lib pendant l'dition de liens
#endif

/* Dfinition des boolens pour C */
#ifndef bool
#define bool unsigned char
#define true 1 == 1
#define false !true
#endif

#define ESC 27			// code ascii de la touche escape

/* Fullscreen */
static bool g_fullscreen = false;

/* Variable contenant le temps au moment de l'affichage de la
 * dernire image, cet variable permet de baser les
 * mouvements sur le temps et non pas sur la vitesse d'affichage
 * de votre ordinateur */
static int g_timeAtLastFrame = 0;

/* Positions des lumires rouge, verte et bleu */
static float lightPositionR[] = { 0.0f, 0.0f, 75.0f, 1.0f };
static float lightPositionG[] = { 0.0f, 0.0f, 75.0f, 1.0f };
static float lightPositionB[] = { 0.0f, 0.0f, 75.0f, 1.0f };

/* Intensits de la lumire ambiante pour les lumires rouge, verte et bleu */
static float diffuseLightR[] = { 1.0f, 0.0f, 0.0f, 1.0f };
static float diffuseLightG[] = { 0.0f, 1.0f, 0.0f, 1.0f };
static float diffuseLightB[] = { 0.0f, 0.0f, 1.0f, 1.0f };

/* Intensits de la lumire spculaire pour les lumires rouge, verte et bleu */
static float specularLightR[] = { 1.0f, 0.0f, 0.0f, 1.0f };
static float specularLightG[] = { 0.0f, 1.0f, 0.0f, 1.0f };
static float specularLightB[] = { 0.0f, 0.0f, 1.0f, 1.0f };

/* Proprits de la lumire gnrale: diffuse, spculaire et position */
static float diffuseLight[] = { 0.5f, 0.5f, 0.5f, 1.0f };	 
static float specularLight[] = { 1.0f, 1.0f, 1.0f, 1.0f };	 
static float lightPosition[] = { 0.0f, 0.0f, 100.0f, 1.0f };

/* Angles de rotation de l'objet clair */
static float objectXRot;
static float objectYRot;
static float objectZRot;

/* Angles de rotation de la lumire rouge */
static float redXRot;
static float redYRot;
/* Angles de rotation de la lumire verte */
static float greenXRot;
static float greenYRot;
/* Angles de rotation de la lumire bleue */
static float blueXRot;
static float blueYRot;	

/* Type de l'objet: 1=Cube, 2=Sphre, 3=Tore */
static int object = 1;	

/* Mode pause */
static bool pause = false;
/* Activation de la lumire rouge */
static bool red = true;
/* Activation de la lumire verte */
static bool green = true;
/* Activation de la lumire bleue */
static bool blue = true;
/* Activation de la rotation de l'objet */
static bool objectRotate = true;	

/* Prototypes */
void init(void);
void askFullscreen(void);
void glutDisplay(void);
void glutIdle(void);
void glutReshape(int w, int h);
void glutKeyboard(unsigned char key, int x, int y);
void terminateApplication(void);

/* Main */
int main(int argc, char ** argv)
{	
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB);
	#ifdef _WIN32
	askFullscreen();
	#endif
	if (!g_fullscreen) {
		glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT);
		glutInitWindowPosition(WINDOW_POS_X, WINDOW_POS_Y);
		glutCreateWindow(TITLE);
	}
	init();
	glutDisplayFunc(glutDisplay);
	glutIdleFunc(glutIdle);
	glutReshapeFunc(glutReshape);
	glutKeyboardFunc(glutKeyboard);
	glutMainLoop();
	return 0;
}

/* Permet de passer en mode plein ecran ou en mode fenetre
 * Utilise GLUT_GAME_MODE disponible uniquement sous Windows
 */
#ifdef _WIN32
void askFullscreen(void)
{
	char gameMode[14];
	
	/* Demande  l'utilisateur s'il veut le mode plein cran */
	if (!g_fullscreen)
	g_fullscreen = (MessageBox(NULL, "Click Yes to go to full screen (Recommended)", "Start FullScreen?",
						MB_ICONQUESTION | MB_YESNO) == IDYES);
	
	
	if (g_fullscreen) {	// game mode (fullscreen)
		sprintf(gameMode, "%dx%d:%d", FULLSCREEN_WIDTH, FULLSCREEN_HEIGHT, FULLSCREEN_DEPTH);
		glutGameModeString(gameMode);
		// teste si le game mode est possible
		if (glutGameModeGet(GLUT_GAME_MODE_POSSIBLE))
		{
			glutEnterGameMode();
			//glutSetCursor(GLUT_CURSOR_NONE);
		}
		else {
			g_fullscreen = false;
		}
	} // mode fentr
}
#endif

/* Initialisation */
void init(void)
{
	/* Affiche des informations sur OpenGL */
	char *exten;
	exten = (char *) glGetString(GL_VENDOR);
	printf("%s\n",exten);
	exten = (char *) glGetString(GL_RENDERER);
	printf("%s\n",exten);
	exten = (char *) glGetString(GL_VERSION);
	printf("%s\n",exten);
	
	printf("\nTouches :\n");
	printf("\tp - Mode pause\n");
 	printf("\tr - Activation/Arret de la lumiere rouge\n");
 	printf("\tg - Activation/Arret de la lumiere verte\n");
 	printf("\tb - Activation/Arret de la lumiere bleue\n");
 	printf("\ts - Activation/Arret de la rotation de l'objet\n");
 	printf("\t1 - l'objet est un cube\n");
 	printf("\t2 - l'objet est une sphere\n");
 	printf("\t3 - l'objet est un tore\n");
	printf("\tESC - Quitter\n");
	
	g_timeAtLastFrame = glutGet(GLUT_ELAPSED_TIME);
	
	/* Initialisation d'OpenGL */
	glClearColor(0.0, 0.0, 0.0, 0.0);

	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);	
	glFrontFace(GL_CCW);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	glEnable(GL_LIGHTING);
	
	// LIGHT0 est spot clairant l'objet  partir du point (0, 0, 100)
	// suivant le vecteur (0, 0, -1)
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
	glLightfv(GL_LIGHT0, GL_SPECULAR, specularLight);
	glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
	glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 40.0f);
	glLightf(GL_LIGHT0, GL_SPOT_EXPONENT, 80.0f);

	// LIGHT1 est la lumire rouge tournant autour de l'objet
	glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuseLightR);
	glLightfv(GL_LIGHT1, GL_SPECULAR, specularLightR);
	glLightfv(GL_LIGHT1, GL_POSITION, lightPositionR);
	
	// LIGHT1 est la lumire verte tournant autour de l'objet
	glLightfv(GL_LIGHT2, GL_DIFFUSE, diffuseLightG);
	glLightfv(GL_LIGHT2, GL_SPECULAR, specularLightG);
	glLightfv(GL_LIGHT2, GL_POSITION, lightPositionG);
	
	// LIGHT1 est la lumire bleue tournant autour de l'objet
	glLightfv(GL_LIGHT3, GL_DIFFUSE, diffuseLightB);
	glLightfv(GL_LIGHT3, GL_SPECULAR, specularLightB);
	glLightfv(GL_LIGHT3, GL_POSITION, lightPositionB);

	// Activation du spot de lumire gnral
	glEnable(GL_LIGHT0);

	// Pour les proprits ambiante et diffuse du matire
	// on utilise les mmes proprits que la couleur active
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

	// On utilise une matire trs brillante
	glMaterialfv(GL_FRONT, GL_SPECULAR, specularLight);
	glMateriali(GL_FRONT, GL_SHININESS, 128);
}
		
/* Affichage et animation */
void glutDisplay(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();										

	// on dplace tous les objets pour qu'ils soient visibles
	glTranslatef(0.0f, 0.0f, -190.0f);
	
	// lumire rouge
	glPushMatrix();
		// rotation de la lumire
		glRotatef(redYRot, 0.0f, 1.0f, 0.0f);
		glRotatef(redXRot, 1.0f, 0.0f, 0.0f);

		// positionnement de la lumire
		glLightfv(GL_LIGHT1, GL_POSITION, lightPositionR);

		// On se positionne  l'endroit o est pla la lumire
		glTranslatef(lightPositionR[0], lightPositionR[1], lightPositionR[2]);
		glColor3f(1.0f, 0.0f, 0.0f);	// couleur rouge
		
		glPushAttrib(GL_LIGHTING_BIT);
			glDisable(GL_LIGHTING);	// Dsactive les effets de lumires
				// Dessine une sphre reprsentant la position de la lumire
				glutSolidSphere(2.5, 20, 15);
			glEnable(GL_LIGHTING);
		glPopAttrib();	
	glPopMatrix();
	
	// lumire verte
	glPushMatrix();
		// rotation de la lumire
		glRotatef(greenYRot, 0.0f, 1.0f, 0.0f);
		glRotatef(greenXRot, 1.0f, 0.0f, 0.0f);

		// positionnement de la lumire
		glLightfv(GL_LIGHT2, GL_POSITION, lightPositionG);
	
		// On se positionne  l'endroit o est pla la lumire
		glTranslatef(lightPositionG[0], lightPositionG[1], lightPositionG[2]);
		glColor3f(0.0f, 1.0f, 0.0f);	// couleur verte
		
		glPushAttrib(GL_LIGHTING_BIT);
			glDisable(GL_LIGHTING);		// Dsactive les effets de lumires
				// Dessine une sphre reprsentant la position de la lumire
				glutSolidSphere(2.5, 20, 15);
			glEnable(GL_LIGHTING);
		glPopAttrib();	
	glPopMatrix();
	
	// lumire bleue
	glPushMatrix();
		// rotation de la lumire
		glRotatef(blueYRot, 0.0f, 1.0f, 0.0f);
		glRotatef(blueXRot, 1.0f, 0.0f, 0.0f);

		// positionnement de la lumire
		glLightfv(GL_LIGHT3, GL_POSITION, lightPositionB);
	
		// On se positionne  l'endroit o est pla la lumire
		glTranslatef(lightPositionB[0], lightPositionB[1], lightPositionB[2]);
		glColor3f(0.0f, 0.0f, 1.0f);	// couleur bleue
		
		glPushAttrib(GL_LIGHTING_BIT);
			glDisable(GL_LIGHTING);		// Dsactive les effets de lumires
				// Dessine une sphre reprsentant la position de la lumire
				glutSolidSphere(2.5, 20, 15);
			glEnable(GL_LIGHTING);
		glPopAttrib();	
	glPopMatrix();
	
	// Dessin de l'objet
	glPushMatrix();
		// Rotation de l'objet
		glRotatef(objectXRot, 1.0f, 0.0f, 0.0f);
		glRotatef(objectYRot, 0.0f, 1.0f, 0.0f);
		glRotatef(objectZRot, 0.0f, 0.0f, 1.0f);
		
		glColor3f(1.0, 1.0, 1.0);
		switch(object) {
			case 1: glutSolidCube(70.0f);
				break;
			case 2: glutSolidSphere(40.0, 30, 30);
				break;	
			case 3: glutSolidTorus(18.0, 35.0, 30, 30);
				break;
		}
	glPopMatrix();
		
	glFlush();
	glutSwapBuffers();
}

/* Fonction excut lorsque le programme n'a plus rien  faire */
void glutIdle(void)
{
	int currentTime = glutGet(GLUT_ELAPSED_TIME);
	int frameInterval = currentTime - g_timeAtLastFrame; // Temps coul entre l'affichage de 2 images
	g_timeAtLastFrame = currentTime;
	
	// Icrmentations des angles de rotation des lumires et de l'objet
	if (!pause) {
		if (objectRotate) {
			objectXRot += (float)frameInterval / 30;
			objectYRot += (float)frameInterval / 25;
			objectZRot += (float)frameInterval / 30;
		}
		if (red) {	
			glEnable(GL_LIGHT1);
			redXRot += (float)frameInterval / 30;
			redYRot += (float)frameInterval / 20;
		}
		else glDisable(GL_LIGHT1);
		if (green) {
			glEnable(GL_LIGHT2);
			greenXRot += (float)frameInterval / 20;
			greenYRot += (float)frameInterval / 30;
		}
		else glDisable(GL_LIGHT2);
		if (blue) {
			glEnable(GL_LIGHT3);
			blueXRot += (float)frameInterval / 30;
			blueYRot += (float)frameInterval / 30;
		}
		else glDisable(GL_LIGHT3);
		
		glutPostRedisplay();	
	}
}

/* Redimensionnement de la fentre */
void glutReshape(int w, int h)
{
	if (h == 0) h = 1;
	glViewport(0, 0, (GLsizei)w, (GLsizei)h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(54.0f,(GLfloat)w/(GLfloat)h, 1.0f, 500.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

/* Gestion du clavier */
void glutKeyboard(unsigned char key, int x, int y)
{
	switch(key) {
		case ESC:	// touche escape
			terminateApplication();
			break;
		case 'P':
		case 'p':	// mode pause
			pause = !pause;
			break;
		case 'R':
		case 'r':	// activation/arrt de la lumire rouge
			red = !red;
			break;
		case 'G':
		case 'g':	// activation/arrt de la lumire verte
			green = !green;
			break;
		case 'B':
		case 'b':	// activation/arrt de la lumire bleue
			blue = !blue;
			break;
		case 'S':
		case 's':	// activation/arrt de la rotation de l'objet
			objectRotate = !objectRotate;
			break;
			
		case '1': object = 1; break;	// l'objet est un cube
		case '2': object = 2; break;	// l'objet est une sphre
		case '3': object = 3; break;	// l'objet est un tore
	}
}

/* Termine l'application */
void terminateApplication(void)
{
	#ifdef _WIN32
	if (g_fullscreen) glutLeaveGameMode();
	#endif
	printf("\nAuteur: Thierry Vouriot\nhttp://thierry.vouriot.free.fr\n");
	exit(1);
}



